def handle_errors(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            raise ValueError(f"Error in {func.__name__}: {str(e)}")

    return wrapper


def handle_errors_for_class(cls):
    for attr_name in dir(cls):
        attr = getattr(cls, attr_name)
        if callable(attr) and not attr_name.startswith("__"):
            setattr(cls, attr_name, handle_errors(attr))
    return cls





def getVectorType(vector):
    if not vector:
        return "null"

    types = set()
    for item in vector:
        types.add(type(item).__name__)

    if len(types) == 1:
        return types.pop()
    else:
        return "mixed"



import matplotlib.pyplot as plt
import numpy as np
from matplotlib.patches import Polygon


class PlotPolygon:
    def __init__(self):
        pass

    @staticmethod
    def plotConvexHull(dots: np.array, **kwargs) -> None:

        """
        imp from : https://stackoverflow.com/questions/43971259/how-to-draw-polygons-with-python
        """
        q = Polygon(kwargs['hull'], facecolor='lightgreen', edgecolor='lightgreen')
        p = Polygon(dots, facecolor='pink', edgecolor='black')
        _, ax = plt.subplots()
        ax.add_patch(q)
        ax.add_patch(p)
        ax.set_xlim([kwargs['min_x'] - 1, kwargs['max_x'] + 1])
        ax.set_ylim([kwargs['min_y'] - 1, kwargs['max_y'] + 1])

        for [x, y] in dots:
            plt.scatter(x, y, color='black')
        for [x, y] in kwargs['hull']:
            plt.scatter(x, y, color='black')
        plt.show()

    def plotPolygons(self, poly: np.array, sub_polygons: np.array) -> None:

        p = Polygon(poly, facecolor='pink', edgecolor='black')
        _, ax = plt.subplots()
        ax.add_patch(p)
        max_x, min_x, max_y, min_y = self.getBoundingBox(poly)

        ax.set_xlim([min_x - 1, max_x + 1])
        ax.set_ylim([min_y - 1, max_y + 1])

        for sub_poly in sub_polygons:
            for edge in sub_poly:
                plt.plot([edge.origin.x, edge.target.x], [edge.origin.y, edge.target.y], color='black')
        plt.show()

    def plotTriangulation(self, Poly: np.array, Triangles: np.array) -> None:
        p = Polygon(Poly, facecolor='pink', edgecolor='black')
        _, ax = plt.subplots()
        ax.add_patch(p)

        max_x, min_x, max_y, min_y = self.getBoundingBox(Poly)

        ax.set_xlim([min_x - 1, max_x + 1])
        ax.set_ylim([min_y - 1, max_y + 1])

        for [x, y] in Poly:
            plt.scatter(x, y, color='black')
        for [x, y] in Triangles:
            plt.plot([x.x, y.x], [x.y, y.y], color='blue', marker='|')
        plt.show()

    @staticmethod
    def getBoundingBox(dots):
        x = [x for (x, _) in dots]
        y = [y for (_, y) in dots]

        return max(x), min(x), max(y), min(y)


def main(_tests, titles=None):
    for test1, test2 in _tests:
        _plt = PlotPolygon()
        max_x, min_x, max_y, min_y = _plt.getBoundingBox(test1)

        kwargs = {'max_x': max_x, 'min_x': min_x, 'max_y': max_y, 'min_y': min_y, 'hull': test2, 'titles': titles}
        PlotPolygon().plotConvexHull(test1, **kwargs)


if __name__ == "__main__":
    tests = [
        [[1, 1], [-1, -1], [1, -1]],
        [[0, 1], [1, 4], [6, 2], [1, 2], [0.5, 1.5]],
        [[0, 0], [0, 1], [1, 1], [1, 0]],
        [[0, 0], [1, 1], [0, 1], [1, 0]],
    ]
    main(tests)



from cgeo.modules.segment_2d import Segment2D as Segment, Point

# Constructors
s1 = Segment()
p = Point(1, 1)
q = Point(2, 2)
s2 = Segment(p, q)

# Output segments
print("s1 : {}".format(s1))
print("s2 : {}\n".format(s2))

# Change origin and target of segment
new_origin1, new_target1 = Point(0, 0), Point(1, 1)
s1.origin, s1.target = new_origin1, new_target1
new_origin2, new_target2 = Point(1, 1), Point(1, 2)
s2.origin, s2.target = new_origin2, new_target2

# Output segments
print("After changing properties: ")
print("s1 : {}".format(s1))
print("s2 : {}\n".format(s2))

# Print general info
print("General properties and methods use: ")
print("s2 length : {}".format(s2.length))
print("s2 slope : {}".format(s2.slope))
s2.rotate(90)
print("s2 slope after rotating with 90 degrees : {}".format(s2.slope))
print("s2 : {}\n".format(s2))

# Orientation predict
print("collide s1 and s2 origin in order to use predict")
null_origin = Point()
s1.origin = null_origin
s2.origin = null_origin
print("s1 : {}".format(s1))
print("s2 : {}\n".format(s2))
print("orientation for s2->s1 is {} (positive)".format(s2.oriePred(s1)))
print("orientation for s1->s2 is {} (negative)\n".format(s1.oriePred(s2)))



from cgeo.modules.point_2d import Point2D as Point
from copy import copy

# constructors
p1 = Point()
p2 = Point(1, 2)

print("test")
print(p2)

print("p1 : {}".format(p1))
print("p2 : {}\n".format(p2))
# set x, y
p1.x = 2
p1.y = 1

print("p1 after setting other coordinates : {}\n".format(p1))

# copy details from another point
p2 = copy(Point(10, 10))

print("p2 after copy coordinates from other point : {}\n".format(p2))

# calculate norm and distance between two points
print("distance between p1 and p2 is {}".format(p2.dist(p1)))
print("p1 norm is {}\n".format(p1.norm))

# compare between two points
print("p1 == p2 is {}".format(p1 == p2))
p2 = copy(p1)
print("(after copying coordinates of p1 to p2)\np1 == p2 is {}\n".format(p1 == p2))


# add and subtract
p2.x = 10
p2.y = 10
p1 = p1.subtract(p2)

print("p1 after subtract p1-p2\np1 : {}".format(p1))
p1 = p1.adder(p2)
print("p1 after adding p1+p2\np1 : {}\n".format(p1))

# orientation predicator for p = (0, 0), q = (1, 0), r = (1, 1)
p, q, r = Point(), Point(1, 1), Point(2, 1)
print("orientation for p->q and p->r is {} (negative)".format(p.oriePred(q, r)))
print("orientation for p->r and p->q is {} (positivenegative)\n".format(p.oriePred(r, q)))



import math
from typing import Tuple, List
import random
from math import atan2


def toConvexContour(num_vertices: int) -> List[Tuple[float, float]]:
    """
    Port of Valtr algorithm by Sander Verdonschot.

    imp from: https://stackoverflow.com/questions/6758083/how-to-generate-a-random-convex-polygon

    """
    xs = [random.uniform(-10, 10) for _ in range(num_vertices)]
    ys = [random.uniform(-10, 10) for _ in range(num_vertices)]
    xs = sorted(xs)
    ys = sorted(ys)
    minX, *xs, maxX = xs
    minY, *ys, maxY = ys
    vectorsXS = _toVectorsCoordinates(coordinates=xs, minCoordinate=minX, maxCoordinate=maxX)
    vectorsYS = _toVectorsCoordinates(coordinates=ys, minCoordinate=minY, maxCoordinate=maxY)
    random.shuffle(vectorsYS)

    def _toVectorAngle(vector):
        x, y = vector
        return atan2(y, x)

    vectors = sorted(zip(vectorsXS, vectorsYS),
                     key=_toVectorAngle)
    pointX = pointY = 0
    minPolygonX = minPolygonY = 0
    points = []
    for vectorX, vectorY in vectors:
        points.append((pointX, pointY))
        pointX += vectorX
        pointY += vectorY
        minPolygonX = min(minPolygonX, pointX)
        minPolygonX = min(minPolygonY, pointY)
    shiftX, shiftY = minX - minPolygonX, minY - minPolygonY
    return [(x + shiftX, y + shiftY)
            for x, y in points]


def _toVectorsCoordinates(coordinates: List[float], minCoordinate: float, maxCoordinate: float) -> list:
    lastMin = lastMax = minCoordinate
    result = []
    for coordinate in coordinates:
        if _toRandomBoolean():
            result.append(coordinate - lastMin)
            lastMin = coordinate
        else:
            result.append(lastMax - coordinate)
            lastMax = coordinate
    result.extend((maxCoordinate - lastMin,
                   lastMax - maxCoordinate))
    return result


def _toRandomBoolean():
    return random.getrandbits(1)


def generateRandomPolygonPoints(center: Tuple[float, float], avg_radius: float,
                                irregularity: float, spikiness: float,
                                num_vertices: int) -> List[Tuple[float, float]]:
    """
        imp from https://stackoverflow.com/questions/8997099/algorithm-to-generate-random-2d-polygon
    """
    # Parameter check
    if irregularity < 0 or irregularity > 1:
        raise ValueError("Irregularity must be between 0 and 1.")
    if spikiness < 0 or spikiness > 1:
        raise ValueError("Spikiness must be between 0 and 1.")

    irregularity *= 2 * math.pi / num_vertices
    spikiness *= avg_radius
    angle_steps = randomAngleSteps(num_vertices, irregularity)

    # now generate the points
    points_set = set()
    points = []
    angle = random.uniform(0, 2 * math.pi)
    counter = 0
    while counter < num_vertices:
        radius = clip(random.gauss(avg_radius, spikiness), 0, 2 * avg_radius)
        point = (center[0] + radius * math.cos(angle),
                 center[1] + radius * math.sin(angle))
        if point in points_set:
            continue
        points.append(point)
        angle += angle_steps[counter]
        points_set.add(point)
        counter += 1

    return points


def randomAngleSteps(steps: int, irregularity: float) -> List[float]:
    # generate n angle steps
    angles = []
    lower = (2 * math.pi / steps) - irregularity
    upper = (2 * math.pi / steps) + irregularity
    cumsum = 0
    for i in range(steps):
        angle = random.uniform(lower, upper)
        angles.append(angle)
        cumsum += angle

    # normalize the steps so that point 0 and point n+1 are the same
    cumsum /= (2 * math.pi)
    for i in range(steps):
        angles[i] /= cumsum
    return angles


def clip(value, lower, upper):
    return min(upper, max(value, lower))



from .modules import *

# modules
from .modules.circle import (Circle)
from .modules.point_2d import (Point2D)
from .modules.segment_2d import (Segment2D)
from .modules.polygon import (Polygon)
from .modules.triangle import (Triangle)





from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.polygon import Polygon


# init polygon
points_arr = []
for x, y in [(0, 0), (1, 0), (1, 1), (0, 1)]:
    points_arr.append(Point(x, y))
poly = Polygon(points_arr)

# print edges
edges_list = poly.getEdges()
for e in edges_list:
    print(e)

# check if convex
print("\nsquare is convex : {}".format(poly.isConvex))
print("square is y-monotone : {}\n".format(poly.isYMonotone))

print("on the other hand if we construct another non convex nor y-monotone such as : \n")
# construct example polygon
test_poly = Polygon([Point(x, y) for x, y in [(0, 0), (-3, 3), (0, 1), (3, 3)]])

# print edges
edges_list = test_poly.getEdges()
for e in edges_list:
    print(e)


# check if convex
print("\ndalton is not convex : {}".format(test_poly.isConvex))
print("dalton is not y-monotone : {}\n".format(test_poly.isYMonotone))



from cgeo.plots_2d import polygon_plot as PolygonPlot
from cgeo.utilities.c_utilities.global_c_utils import triangulate
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.polygon import Polygon
from cgeo.utilities.get_random_polygon import generateRandomPolygonPoints


plt = PolygonPlot.PlotPolygon()
for i in range(10):
    dots = generateRandomPolygonPoints(center=(0, 0),
                                       avg_radius=20,
                                       irregularity=0.5,
                                       spikiness=0.5,
                                       num_vertices=10)

    points_arr = [Point(x, y) for x, y in dots]
    poly = Polygon(points_arr)

    # decompose to triangles
    Triangles = triangulate(poly)
    edges = []

    for tri in Triangles:
        vertex = tri.getVertices()
        edges.append((vertex[0], vertex[1]))
        edges.append((vertex[1], vertex[2]))
        edges.append((vertex[2], vertex[0]))

    plt.plotTriangulation(Poly=dots, Triangles=edges)




from cgeo.plots_2d import polygon_plot as PolygonPlot
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.polygon import Polygon
from cgeo.utilities.get_random_polygon import generateRandomPolygonPoints

dots = generateRandomPolygonPoints(center=(0, 0),
                                   avg_radius=10,
                                   irregularity=0.2,
                                   spikiness=0.5,
                                   num_vertices=20)

points_arr = [Point(x, y) for x, y in dots]
poly = Polygon(points_arr)

sub_polygons = poly.decomposeToYMonotone()

sub_polygons_points = []
for p in sub_polygons:
    sub_polygons_points.append(p.getEdges())

PolygonPlot.PlotPolygon().plotPolygons(dots, sub_polygons_points)



from __future__ import annotations
from cgeo.utilities.error_handling.error_utilities import *

from cgeo import _cgeo


@handle_errors_for_class
class Point2D:
    def __init__(self, x=0.0, y=0.0):
        self._point = _cgeo.libShapes_2D.Point(x, y)

    @property
    def x(self) -> float:
        return self._point.x

    @x.setter
    def x(self, value: float) -> None:
        self._point.x = value

    @property
    def y(self) -> float:
        return self._point.y

    @y.setter
    def y(self, value: float) -> None:
        self._point.y = value

    def __copy__(self) -> Point2D:
        return Point2D(self.x, self.y)

    def __str__(self) -> str:
        return self._point.toStr()

    def __eq__(self, other: Point2D) -> bool:
        return self._point.__eq__(other._point)

    def __lt__(self, other: Point2D) -> bool:
        return self._point.__lt__(other._point)

    def __gt__(self, other: Point2D) -> bool:
        return self._point.__gt__(other._point)

    def toDict(self) -> dict:
        return {"x": self.x, "y": self.y}

    @classmethod
    def fromDict(cls, data) -> Point2D:
        return cls(data["x"], data["y"])

    @classmethod
    def fromCPoint(cls, cPoint: _cgeo.libShapes_2D.Point):
        newPoint = cls()
        newPoint._point = cPoint
        return newPoint

    @property
    def norm(self) -> float:
        return self._point.norm()

    def dist(self, other: Point2D) -> float:
        return self._point.dist(other._point)

    def subtract(self, other: Point2D) -> Point2D:
        return Point2D(self.x - other.x, self.y - other.y)

    def adder(self, other: Point2D) -> Point2D:
        return Point2D(self.x + other.x, self.y + other.y)

    def oriePred(self, q: Point2D, r: Point2D) -> float:
        return self._point.oriePred(q._point, r._point)

    @property
    def cPoint(self):
        return self._point



from cgeo.modules.polygon import Polygon
from cgeo.modules.segment_2d import Segment2D as Segment
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.triangle import Triangle
from cgeo.modules.circle import Circle

from cgeo import _cgeo

def grahamConvexHull(polygon: Polygon) -> Polygon:
    cPolygon = _cgeo.libAlgorithms_2D.ConvexHullUtils.grahamConvexHull(polygon.cPolygon)
    return Polygon.fromCPolygon(cPolygon)


def giftWrapConvexHull(polygon: Polygon) -> Polygon:
    cPolygon = _cgeo.libAlgorithms_2D.ConvexHullUtils.giftWrapConvexHull(polygon.cPolygon)
    return Polygon.fromCPolygon(cPolygon)


def segmentIntersection(segments: list[Segment]) -> list[Point]:
    cPointList = _cgeo.libAlgorithms_2D.SegmentIntersect.intersection([s.cSegment for s in segments])
    return [Point.fromCPoint(p) for p in cPointList]


def triangulate(polygon: Polygon) -> list[Triangle]:
    cTriangles = _cgeo.libAlgorithms_2D.TriangulationUtils.triangulate(polygon.cPolygon)
    return [Triangle.fromCTriangle(triangle) for triangle in cTriangles]


def enclosingDisc(points: list[Point]) -> Circle:
    cCircle = _cgeo.libAlgorithms_2D.EnclosingDistUtils.findDisc([p.cPoint for p in points])
    return Circle.fromCCircle(cCircle)



from __future__ import annotations
from cgeo.modules.point_2d import Point2D as Point
from cgeo.utilities.error_handling.error_utilities import handle_errors_for_class

from cgeo import _cgeo


@handle_errors_for_class
class Triangle:
    def __init__(self, a: Point, b: Point, c: Point) -> None:
        self._triangle = _cgeo.libShapes_2D.Triangle(a.cPoint, b.cPoint, c.cPoint)

    @property
    def cTriangle(self) -> _cgeo.libShapes_2D.Triangle:
        return self._triangle

    @property
    def a(self) -> Point:
        return Point.fromCPoint(self._triangle.a)

    @a.setter
    def a(self, newPoint: Point) -> None:
        self._triangle.a = newPoint.cPoint

    @property
    def b(self) -> Point:
        return Point.fromCPoint(self._triangle.b)

    @b.setter
    def b(self, newPoint: Point) -> None:
        self._triangle.b = newPoint.cPoint

    @property
    def c(self) -> Point:
        return Point.fromCPoint(self._triangle.c)

    @c.setter
    def c(self, newPoint: Point) -> None:
        self._triangle.c = newPoint.cPoint

    @property
    def area(self) -> float:
        return self._triangle.area()

    @classmethod
    def fromCTriangle(cls, cTriangle: _cgeo.libShapes_2D.Triangle) -> Triangle:
        newTriangle = cls(Point.fromCPoint(cTriangle.a), Point.fromCPoint(cTriangle.b), Point.fromCPoint(cTriangle.c))
        newTriangle._triangle = cTriangle
        return newTriangle

    def getVertices(self) -> (Point, Point, Point):
        return self.a, self.b, self.c



from __future__ import annotations
from typing import Union
from cgeo.modules.point_2d import Point2D as Point
from cgeo.utilities.error_handling.error_utilities import *

from cgeo import _cgeo

@handle_errors_for_class
class Segment2D:
    def __init__(self, *args):
        if len(args) == 0:
            self._segment = _cgeo.libShapes_2D.Segment()
        elif len(args) == 1:
            assert(isinstance(args[0], Segment2D))
            self._segment = _cgeo.libShapes_2D.Segment(args[0].origin, args[0].target)
        elif len(args) == 2:
            assert(isinstance(args[0], Point) and isinstance(args[1], Point))
            self._segment = _cgeo.libShapes_2D.Segment(args[0].cPoint, args[1].cPoint)
        else:
            raise ValueError('too many arguments')

    def __eq__(self, other: Segment2D) -> bool:
        return self._segment.__eq__(other.cSegment)

    def __copy__(self):
        return Segment2D(self.origin, self.target)

    def __str__(self):
        return str(self._segment)

    @property
    def origin(self) -> _cgeo.libShapes_2D.Point:
        return self._segment.origin

    @origin.setter
    def origin(self, value: Point | _cgeo.libShapes_2D.Point) -> None:
        self._segment.origin = value if isinstance(value, _cgeo.libShapes_2D.Point) else value.cPoint

    @property
    def target(self) -> _cgeo.libShapes_2D.Point:
        return self._segment.target

    @target.setter
    def target(self, value: Point | _cgeo.libShapes_2D.Point) -> None:
        self._segment.target = value if isinstance(value, _cgeo.libShapes_2D.Point) else value.cPoint

    @property
    def slope(self) -> float:
        return self._segment.getSlope()

    @property
    def length(self) -> float:
        return self._segment.getLength()

    @property
    def cSegment(self) -> _cgeo.libShapes_2D.Segment:
        return self._segment

    @classmethod
    def fromCSegment(cls, cSegment: _cgeo.libShapes_2D.Segment):
        newSegment = cls()
        newSegment._segment = cSegment
        return newSegment

    def __originX(self) -> float:
        return self.origin.x

    def __originY(self) -> float:
        return self.origin.y

    def __targetX(self) -> float:
        return self.target.x

    def __targetY(self) -> float:
        return self.target.y

    def getUpperPoint(self) -> _cgeo.libShapes_2D.Point:
        cPoint = self._segment.getUpper()
        return self.origin if self.__originX() == cPoint.x and self.__originY() == cPoint.y else self.target

    def adder(self, offsetPoint: Point) -> None:
        self._segment.adder(_cgeo.libShapes_2D.Point(offsetPoint.x, offsetPoint.y))

    def rotate(self, d: float) -> None:
        self._segment.rotate(d)

    def oriePred(self, other: Union[Point, Segment2D]) -> float:
        if not isinstance(other, (Point, Segment2D)):
            raise TypeError('Invalid argument type. Expected Point or Segment2D.')

        if isinstance(other, Point):
            return self._segment.oriePred(other.cPoint)

        return self._segment.oriePred(other.cSegment)

    def isParallel(self, other: Segment2D) -> bool:
        return self._segment.isParallel(other.cSegment)

    def isVertical(self, other: Segment2D) -> bool:
        return self._segment.isVertical(other.cSegment)

    def dist(self, other: Union[Point, Segment2D]) -> float:
        if not isinstance(other, (Point, Segment2D)):
            raise TypeError('Invalid argument type. Expected Point or Segment2D.')

        if isinstance(other, Point):
            return self._segment.dist(other.cPoint)

        return self._segment.dist(other.cSegment)

    def isIntersect(self, other: Segment2D) -> bool:
        return self._segment.isIntersect(other.cSegment)



import random
import cgeo.plots_2d.circle_plot as CirclePlot
from cgeo.modules.point_2d import Point2D as Point
from cgeo.utilities.c_utilities.global_c_utils import enclosingDisc


def generatePoints(n=10):
    _points = []
    for i in range(n):
        _points.append(Point(random.randint(-10, 10), random.randint(-10, 10)))

    return _points


points_vector = generatePoints(50)

disc = enclosingDisc(points_vector)

CirclePlot.PlotCircles().plotEnclosingDisc(points_vector, disc)



from cgeo.plots_2d import polygon_plot as PolygonPlot
from cgeo.utilities.c_utilities.global_c_utils import *
from cgeo.utilities.get_random_polygon import generateRandomPolygonPoints

polygons = []

for i in range(10):
    dots = generateRandomPolygonPoints(center=(0, 0),
                                       avg_radius=20,
                                       irregularity=0.5,
                                       spikiness=0.5,
                                       num_vertices=20)
    points_arr = [Point(x, y) for x, y in dots]
    poly = Polygon(points_arr)
    convex_poly = giftWrapConvexHull(poly)
    polygons.append((poly, convex_poly))

test = []
for poly1, poly2 in polygons:
    curr1 = []
    for p in poly1.getVertices():
        curr1.append([p.x, p.y])
    curr2 = []
    for p in poly2.getVertices():
        curr2.append([p.x, p.y])
    test.append((curr1, curr2))
PolygonPlot.main(_tests=test, titles=["square", "dalton"])



from __future__ import annotations
from cgeo.utilities.get_vector_type import getVectorType
from cgeo.utilities.error_handling.error_utilities import handle_errors_for_class
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.segment_2d import Segment2D as Segment

from cgeo import _cgeo

@handle_errors_for_class
class Polygon:
    def __init__(self, args=None):
        if args is None:
            args = []
        if len(args) == 0:
            self._polygon = _cgeo.libShapes_2D.Polygon()
        elif getVectorType(args) != Point.__name__:
            raise TypeError('invalid argument')
        else:
            self._polygon = _cgeo.libShapes_2D.Polygon([p.cPoint for p in args])

    @property
    def cPolygon(self) -> _cgeo.libShapes_2D.Polygon:
        return self._polygon

    @property
    def size(self) -> int:
        return self._polygon.getSize()

    @property
    def isEmpty(self) -> bool:
        return self.size == 0

    @property
    def isConvex(self) -> bool:
        return self._polygon.isConvex()

    @property
    def isYMonotone(self) -> bool:
        return self._polygon.isY_Monotone()

    @classmethod
    def fromCPolygon(cls, cPolygon: _cgeo.libShapes_2D.Polygon) -> Polygon:
        newPolygon = cls()
        newPolygon._polygon = cPolygon
        return newPolygon

    def getVertices(self) -> list[Point]:
        cVertices = self._polygon.getVertices()
        return [Point.fromCPoint(cVertex) for cVertex in cVertices]

    def insert(self, vertex: Point, index=-1) -> None:
        if index < 0:
            self._polygon.insert(vertex.cPoint)
        else:
            self._polygon.insert(vertex.cPoint, index)

    def getByIndex(self, index: int) -> Point:
        return Point.fromCPoint(self._polygon.getByIndex(index))

    def getLexMin(self) -> Point:
        return Point.fromCPoint(self._polygon.getLexMin())

    def getLexMinIndex(self) -> int:
        return self._polygon.getLexMin_index()

    def getLexMax(self) -> Point:
        return Point.fromCPoint(self._polygon.getLexMax())

    def getLexMaxIndex(self) -> int:
        return self._polygon.getLexMax_index()

    def getLexOrder(self) -> list[Point]:
        return [Point.fromCPoint(cVertex) for cVertex in self._polygon.lexOrder()]

    def getEdges(self) -> list[Segment]:
        return [Segment.fromCSegment(cEdge) for cEdge in self._polygon.getEdges()]

    def isInnerCusp(self, index: int) -> bool:
        return self._polygon.isInnerCusp(index)

    def decomposeToYMonotone(self) -> list[Polygon]:
        return [Polygon.fromCPolygon(cPolygon) for cPolygon in self._polygon.decomposeY_Monotone()]

    def toList(self) -> list:
        return [[p.x, p.y] for p in self.getVertices()]



import matplotlib.pyplot as plt
from matplotlib import patches


class PlotCircles:
    def __init__(self):
        pass

    @staticmethod
    def plotEnclosingDisc(dots, circle) -> None:

        fig = plt.figure()
        ax = fig.add_subplot()
        circle1 = patches.Circle((circle.center.x, circle.center.y), radius=circle.r, edgecolor='blue', fill=False)
        ax.add_patch(circle1)
        ax.axis('equal')

        for dot in dots:
            plt.scatter(dot.x, dot.y, color='black', s=5)
        plt.grid()
        plt.show()


import os
import sys
DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(DIR, "extern", "pybind11"))

from glob import glob
from setuptools import setup, find_packages
from pybind11.setup_helpers import Pybind11Extension, build_ext  # noqa: E402

del sys.path[-1]
pkg_name = 'cgeo'
ext_name = '_cgeo'
__version__ = '0.1.2'
__longDescription__ = open("README.md").read()

# don't forget to maintain while adding more modules
__pyModules__ = ['Point2D', 'Circle', 'Polygon', 'Segment2D', 'Triangle']

def srcFiles(directory):
    pattern = directory + '/**/*.cpp'
    return sorted(glob(pattern, recursive=True))

setup(
    name=pkg_name,
    version=__version__,

    description='Open source package for computational geometry',
    long_description_content_type='text/markdown',
    long_description=__longDescription__,

    author='Tom Sabala',
    author_email='sabala144@gmail.com',

    packages=find_packages(),
    package_data={pkg_name: ['./**.*']},
    py_modules=__pyModules__,

    ext_modules=[
        Pybind11Extension(
            name= '{}.{}'.format(pkg_name, ext_name),
            include_dirs=['./src'],
            sources=srcFiles('src'),
            define_macros=[('EXTENSION_NAME', ext_name)],
            extra_compile_args = ['-O3', '-Wall'],
        ),
    ],

    cmdclass={"build_ext": build_ext},

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: Unix",
    ],
)


from __future__ import annotations
from cgeo.utilities.error_handling.error_utilities import handle_errors_for_class
from cgeo.modules.segment_2d import Segment2D as Segment
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.triangle import Triangle
from cgeo.modules.polygon import Polygon

from cgeo import _cgeo


@handle_errors_for_class
class Circle:
    def __init__(self, r=0.0, center=Point(0, 0)):
        self._circle = _cgeo.libShapes_2D.Circle(r, center.cPoint)

    @classmethod
    def fromTriangle(cls, triangle: Triangle) -> Circle:
        newCircle = cls()
        cCircle = _cgeo.libShapes_2D.Circle(triangle.a.cPoint, triangle.b.cPoint, triangle.c.cPoint)
        newCircle._circle = cCircle
        return newCircle

    @property
    def r(self) -> float:
        return self._circle.r

    @property
    def center(self) -> Point:
        return Point.fromCPoint(self._circle.center)

    @property
    def cCircle(self) -> _cgeo.libShapes_2D.Circle:
        return self._circle

    @classmethod
    def fromCCircle(cls, cCircle: _cgeo.libShapes_2D.Circle) -> Circle:
        newCircle = cls()
        newCircle._circle = cCircle
        return newCircle

    def circleIntersect(self, circle: Circle) -> bool:
        return self._circle.circleIntersect(circle.cCircle)

    # should add line intersect

    def segIntersect(self, segment: Segment) -> bool:
        return self._circle.segIntersect(segment.cSegment)

    def polyIntersect(self, polygon: Polygon) -> bool:
        return self._circle.polyIntersect(polygon.cPolygon)

    def pointContains(self, point: Point) -> bool:
        return self._circle.pointContains(point.cPoint)

    def segmentContains(self, segment: Segment) -> bool:
        return self._circle.segmentContains(segment.cSegment)

    def circleContains(self, circle: Circle) -> bool:
        return self._circle.circleContains(circle.cCircle)

    def polyContains(self, polygon: Polygon) -> bool:
        return self._circle.polyContains(polygon.cPolygon)



from cgeo.plots_2d import segments_plot as SegmentsPlot
from cgeo.modules.point_2d import Point2D as Point
from cgeo.modules.segment_2d import Segment2D as Segment
from cgeo.utilities.c_utilities.global_c_utils import segmentIntersection

segments_coord = SegmentsPlot.generate_random_segments(20)

segments_test = []
for i, seg in enumerate(segments_coord):
    p = Point(seg[0][0], seg[0][1])
    q = Point(seg[1][0], seg[1][1])
    seg = Segment(p, q)
    segments_test.append(seg)

try:
    intersection_points = segmentIntersection(segments_test)

    seg2plot = [[[seg.target.x, seg.target.y], [seg.origin.x, seg.origin.y]] for seg in segments_test]

    inter2plot = [[point.x, point.y] for point in intersection_points]

    SegmentsPlot.main(seg2plot, inter2plot)
except Exception as e:
    print(e)



import matplotlib.pyplot as plt
import numpy as np


class PlotSegments:
    def __init__(self):
        pass

    @staticmethod
    def plot(xy: np.array, dots, **kwargs) -> None:
        fig, ax = plt.subplots(figsize=(10, 10))
        ab_pairs = np.c_[xy[0], xy[1]]
        ab_args = ab_pairs.reshape(-1, 2, 2).swapaxes(1, 2).reshape(-1, 2)

        # segments
        ax.plot(*ab_args, c='k')
        for dot in dots:
            plt.scatter([dot[0]], [dot[1]], color='blue')
        plt.grid()
        plt.show()


def generate_random_segments(n: int) -> list:
    import random

    segments = []
    for i in range(n):
        x1, y1 = random.randint(0, 20), random.randint(0, 20)
        x2, y2 = random.randint(0, 20), random.randint(0, 20)

        while x1 == x2 and y1 == y2:
            x1, y1 = random.randint(0, 20), random.randint(0, 20)
            x2, y2 = random.randint(0, 20), random.randint(0, 20)

        segments.append([[x1, y1], [x2, y2]])

    return segments


def main(segments, dots):
    _x = []
    _y = []
    for seg in segments:
        _x.append(seg[0][0])
        _x.append(seg[1][0])
        _y.append(seg[0][1])
        _y.append(seg[1][1])
    kwargs = {'max_x': max(_x), 'min_x': min(_x), 'max_y': max(_y), 'min_y': min(_y)}

    PlotSegments().plot([_x, _y], dots, **kwargs)
